var targetId = "WebObjectTree1";
var controlIds = ["TextBox1", "Button1", "Button2", "Button3", "Button4", "Button5"];
var customData = "";

// Displays the webobjecttree selected full path
function CustomDisplayFullPath(controlId, fullPathId)
{
    var innerKey = "\"innerKey\"";
    var originalLength = customData.length;
    var replacedLength = customData.replace(/\"innerKey\"/g, "").length;
    var nodeCount = (originalLength - replacedLength) / innerKey.length - 1;

    var fullPath = document.getElementById(fullPathId);
    fullPath.innerHTML = cwotGetFullPath(controlId) + "<br />Node count = " + nodeCount + ".";
}

// Sets the enabled state of the textbox/buttons
function SetEnabledState(enabled)
{
    var empty = IsTreeEmpty();
    if (empty)
    {
        var textBox = document.getElementById(controlIds[0]);
        textBox.value = "";
    }
    var length = controlIds.length;
    for (var index = 0; index < length; index++)
    {
        var element = document.getElementById(controlIds[index]);
        if (empty && index != 1)
        {
            element.disabled = true;
        }
        else
        {
            element.disabled = !enabled;
        }
    }
}

// Gets the path to the parent node
function GetParentPath(childPath)
{
    var parentPath = "/";
    var index = childPath.lastIndexOf("/");
    if (index > 0)
    {
        parentPath = childPath.substring(0, index);
    }

    return parentPath;
}

// Gets the path to the child node
function GetChildPath(parentPath, childNode)
{
    var childPath = cwotAppendKeyValue(parentPath, childNode.innerKey, childNode.innerText);

    return childPath
}

// Gets the target node and its elder relatives
function GetNodeInfo(targetPath)
{
    var keyPath = cwotExtractKeys(targetPath);
    var nodeInfo = FindNode(keyPath);

    return nodeInfo;
}

// Gets the requested children items
function GetChildren(requestPath)
{
    var items = [];
    var nodeInfo = GetNodeInfo(requestPath);
    var node = nodeInfo.targetNode.firstChild;
    while (node != null)
    {
        var key = node.innerKey;
        var text = node.innerText;
        var keyValue = "[" + key + "]" + text;
        var children = (node.firstChild != null);

        var item =
        {
            Item: keyValue,
            Children: children
        }

        items.push(item);
        node = node.nextSibling;
    }

    var results =
    {
        Converter: "KeyValue",
        Items: items
    };

    return results;
}

// Refreshes the children items
function RefreshChildren(clearPath, selectedPath)
{
    cwotClearChildren(targetId, clearPath);
    if (selectedPath != null)
    {
        cwotSetFullPath(targetId, selectedPath);
        cwotSetFocus(targetId, false);
    }
}

// Retrieves data requested by the webobjecttree control
function cwotOnBeginRequest(controlId, requestPath)
{
    var responseData = null;
    if (controlId == targetId)
    {
        if (treeData.rootNode == null)
        {
            customData = cwotGetCustomData(targetId);
            DeserializeTree(customData);
        }
        responseData = GetChildren(requestPath);
    }

    return responseData;
}

// Disables the textbox/buttons when data retrieval starts
function cwotOnDataRetrievalBegin(controlId)
{
    if (controlId == targetId)
    {
        SetEnabledState(false);
    }
}

// Enables the textbox/buttons when data retrieval stops
function cwotOnDataRetrievalEnd(controlId)
{
    if (controlId == targetId)
    {
        SetEnabledState(true);
    }
}

// Updates the textbox when the user changes the selected item
function cwotOnSelectionChange(controlId, selectedValue)
{
    if (controlId == targetId)
    {
        var textBox = document.getElementById(controlIds[0]);
        textBox.value = cwotExtractValues(selectedValue);
    }
}

// Allows the user to rename the current item
function cwotOnAllowRename(controlId, renamePath)
{
    var allowRename = false;
    if (controlId == targetId)
    {
        allowRename = true;
    }

    return allowRename;
}

// Handles the renaming of the current item
function cwotOnRenameItem(controlId, renamePath, newValue)
{
    var renameResult = false;
    if (controlId == targetId)
    {
        var nodeInfo = GetNodeInfo(renamePath);
        nodeInfo.targetNode.innerText = newValue;
        var textBox = document.getElementById(controlIds[0]);
        textBox.value = newValue;
        renameResult = true;
    }

    return renameResult;
}

// Allows the user to delete using the keyboard
function cwotOnKeyDown(controlId, eventObject)
{
    if (controlId == targetId && eventObject.keyCode == 46)
    {
        var empty = IsTreeEmpty();
        if (!empty)
        {
            OnDeleteNode();
        }
    }

    return true;
}

// Adds a new child node
function OnAddChild()
{
    var newNode = CreateNode(null);
    var targetPath = cwotGetFullPath(targetId);
    var nodeInfo = GetNodeInfo(targetPath);
    if (nodeInfo.targetNode.firstChild == null)
    {
        nodeInfo.targetNode.firstChild = newNode;
    }
    else
    {
        var lastChild = GetLastChild(nodeInfo.targetNode);
        lastChild.nextSibling = newNode;
    }
    var childPath = GetChildPath(targetPath, newNode);
    RefreshChildren(targetPath, childPath);
}

// Adds a new sibling node
function OnAddSibling()
{
    var newNode = CreateNode(null);
    var targetPath = cwotGetFullPath(targetId);
    var nodeInfo = GetNodeInfo(targetPath);
    var lastChild = GetLastChild(nodeInfo.parentNode);
    lastChild.nextSibling = newNode;
    var parentPath = GetParentPath(targetPath);
    var childPath = GetChildPath(parentPath, newNode);
    RefreshChildren(parentPath, childPath);
}

// Modifies the current node
function OnModifyNode()
{
    var targetPath = cwotGetFullPath(targetId);
    var nodeInfo = GetNodeInfo(targetPath);
    var textBox = document.getElementById(controlIds[0]);
    if (nodeInfo.targetNode.innerText != textBox.value)
    {
        cwotRenameItem(targetId, targetPath, textBox.value);
        nodeInfo.targetNode.innerText = textBox.value;
    }
}

// Deletes the current node
function OnDeleteNode()
{
    var targetPath = cwotGetFullPath(targetId);
    var nodeInfo = GetNodeInfo(targetPath);
    if (nodeInfo.parentNode.firstChild == nodeInfo.targetNode)
    {
        nodeInfo.parentNode.firstChild = nodeInfo.targetNode.nextSibling;
    }
    else
    {
        nodeInfo.previousSibling.nextSibling = nodeInfo.targetNode.nextSibling;
    }
    var parentPath = GetParentPath(targetPath);
    var childPath = targetPath;
    if (nodeInfo.targetNode.nextSibling != null)
    {
        childPath = GetChildPath(parentPath, nodeInfo.targetNode.nextSibling);
    }
    else if (nodeInfo.previousSibling != null)
    {
        childPath = GetChildPath(parentPath, nodeInfo.previousSibling);
    }
    else if (treeData.rootNode.firstChild == null)
    {
        ClearTree();
        SetEnabledState(true);
        childPath = null;
    }
    RefreshChildren(parentPath, childPath);
}

// Deletes all nodes
function OnDeleteAll()
{
    ClearTree();
    SetEnabledState(true);
    RefreshChildren("/", null);
}

// Stores the tree data for postback
function OnStoreTree()
{
    customData = SerializeTree();
    cwotSetCustomData(targetId, customData);
    var textBox = document.getElementById(controlIds[0]);
    textBox.value = textBox.value.replace(/</g, "&lt;");
}

// Initializes the webobjecttree control on the page
function CustomWebObjectTreeInitialize(controlId)
{
    WebObjectTreeInitialize(controlId);
    cwotSetFullPath(controlId, "/KeyValue:[ID1]New Item 1/KeyValue:[ID2]New Item 2");
}
